@@ -8,7 +8,8 @@
 #include <string.h>
 #include <unistd.h>
 
-char *os_release = "/etc/os-release";
+static const char *os_release = "/etc/os-release";
+static const char *meta_snap_yaml = "/meta/snap.yaml";
 
 sc_distro sc_classify_distro(void)
 {
@@ -38,6 +39,14 @@ sc_distro sc_classify_distro(void)
 		}
 	}
 
+	if (!is_core) {
+		/* Since classic systems don't have a /meta/snap.yaml file the simple
+		   presence of that file qualifies as SC_DISTRO_CORE_OTHER. */
+		if (access(meta_snap_yaml, F_OK) == 0) {
+			is_core = true;
+		}
+	}
+
 	if (is_core) {
 		if (core_version == 16) {
 			return SC_DISTRO_CORE16;
