@@ -34,6 +34,10 @@
 #include "../libsnap-confine-private/string-utils.h"
 #include "../libsnap-confine-private/utils.h"
 
+// SANITY_TIMEOUT is the timeout in seconds that is used when
+// "sc_enable_sanity_timeout()" is called
+static const int SANITY_TIMEOUT = 30;
+
 /**
  * Flag indicating that a sanity timeout has expired.
  **/
@@ -61,8 +65,9 @@ void sc_enable_sanity_timeout(void)
 	if (sigaction(SIGALRM, &act, NULL) < 0) {
 		die("cannot install signal handler for SIGALRM");
 	}
-	alarm(6);
-	debug("sanity timeout initialized and set for three seconds");
+	alarm(SANITY_TIMEOUT);
+	debug("sanity timeout initialized and set for %i seconds",
+	      SANITY_TIMEOUT);
 }
 
 void sc_disable_sanity_timeout(void)
