@@ -22,6 +22,7 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "utils.h"
 #include "string-utils.h"
@@ -30,7 +31,7 @@
 bool verify_security_tag(const char *security_tag, const char *snap_name)
 {
 	const char *whitelist_re =
-	    "^snap\\.([a-z0-9](-?[a-z0-9])*)\\.([a-zA-Z0-9](-?[a-zA-Z0-9])*|hook\\.[a-z](-?[a-z])*)$";
+	    "^snap\\.([a-z0-9](-?[a-z0-9])*(_[a-z0-9]{1,10})?)\\.([a-zA-Z0-9](-?[a-zA-Z0-9])*|hook\\.[a-z](-?[a-z])*)$";
 	regex_t re;
 	if (regcomp(&re, whitelist_re, REG_EXTENDED) != 0)
 		die("can not compile regex %s", whitelist_re);
@@ -56,7 +57,7 @@ bool verify_security_tag(const char *security_tag, const char *snap_name)
 bool sc_is_hook_security_tag(const char *security_tag)
 {
 	const char *whitelist_re =
-	    "^snap\\.[a-z](-?[a-z0-9])*\\.(hook\\.[a-z](-?[a-z])*)$";
+	    "^snap\\.[a-z](-?[a-z0-9])*(_[a-z0-9]{1,10})?\\.(hook\\.[a-z](-?[a-z])*)$";
 
 	regex_t re;
 	if (regcomp(&re, whitelist_re, REG_EXTENDED | REG_NOSUB) != 0)
@@ -97,6 +98,94 @@ static int skip_one_char(const char **p, char c)
 	return 0;
 }
 
+void sc_instance_name_validate(const char *instance_name,
+			       struct sc_error **errorp)
+{
+	// NOTE: This function should be synchronized with the two other
+	// implementations: validate_instance_name and snap.ValidateInstanceName.
+	struct sc_error *err = NULL;
+
+	// Ensure that name is not NULL
+	if (instance_name == NULL) {
+		err =
+		    sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_INSTANCE_NAME,
+				  "snap instance name cannot be NULL");
+		goto out;
+	}
+	// 40 char snap_name + '_' + 10 char instance_key + 1 extra overflow + 1
+	// NULL
+	char s[53] = { 0 };
+	strncpy(s, instance_name, sizeof(s) - 1);
+
+	char *t = s;
+	const char *snap_name = strsep(&t, "_");
+	const char *instance_key = strsep(&t, "_");
+	const char *third_separator = strsep(&t, "_");
+	if (third_separator != NULL) {
+		err =
+		    sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_INSTANCE_NAME,
+				  "snap instance name can contain only one underscore");
+		goto out;
+	}
+
+	sc_snap_name_validate(snap_name, &err);
+	if (err != NULL) {
+		goto out;
+	}
+	// When the instance_name is a normal snap name, instance_key will be
+	// NULL, so only validate instance_key when we found one.
+	if (instance_key != NULL) {
+		sc_instance_key_validate(instance_key, &err);
+	}
+
+ out:
+	sc_error_forward(errorp, err);
+}
+
+void sc_instance_key_validate(const char *instance_key,
+			      struct sc_error **errorp)
+{
+	// NOTE: see snap.ValidateInstanceName for reference of a valid instance key
+	// format
+	struct sc_error *err = NULL;
+
+	// Ensure that name is not NULL
+	if (instance_key == NULL) {
+		err = sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_NAME,
+				    "instance key cannot be NULL");
+		goto out;
+	}
+	// This is a regexp-free routine hand-coding the following pattern:
+	//
+	// "^[a-z]{1,10}$"
+	//
+	// The only motivation for not using regular expressions is so that we don't
+	// run untrusted input against a potentially complex regular expression
+	// engine.
+	int i = 0;
+	for (i = 0; instance_key[i] != '\0'; i++) {
+		if (islower(instance_key[i]) || isdigit(instance_key[i])) {
+			continue;
+		}
+		err =
+		    sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_INSTANCE_KEY,
+				  "instance key must use lower case letters or digits");
+		goto out;
+	}
+
+	if (i == 0) {
+		err =
+		    sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_INSTANCE_KEY,
+				  "instance key must contain at least one letter or digit");
+	} else if (i > 10) {
+		err =
+		    sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_INSTANCE_KEY,
+				  "instance key must be shorter than 10 characters");
+	}
+ out:
+	sc_error_forward(errorp, err);
+}
+
 void sc_snap_name_validate(const char *snap_name, struct sc_error **errorp)
 {
 	// NOTE: This function should be synchronized with the two other
@@ -159,10 +248,17 @@ void sc_snap_name_validate(const char *snap_name, struct sc_error **errorp)
 	if (!got_letter) {
 		err = sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_NAME,
 				    "snap name must contain at least one letter");
+		goto out;
+	}
+	if (n < 2) {
+		err = sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_NAME,
+				    "snap name must be longer than 1 character");
+		goto out;
 	}
 	if (n > 40) {
 		err = sc_error_init(SC_SNAP_DOMAIN, SC_SNAP_INVALID_NAME,
 				    "snap name must be shorter than 40 characters");
+		goto out;
 	}
 
  out:
