@@ -69,67 +69,6 @@ var (
 	ioutilReadDir = ioutil.ReadDir
 )
 
-// IsReadOnly returns true if a directory is ready only.
-//
-// Directories are read only when they reside on file systems mounted in read
-// only mode or when the underlying file system itself is inherently read only.
-func IsReadOnly(dirFd int, dirName string) (bool, error) {
-	var fsData syscall.Statfs_t
-	if err := sysFstatfs(dirFd, &fsData); err != nil {
-		return false, fmt.Errorf("cannot fstatfs %q: %s", dirName, err)
-	}
-	// If something is mounted with f_flags & ST_RDONLY then is read-only.
-	if fsData.Flags&StReadOnly == StReadOnly {
-		return true, nil
-	}
-	// If something is a known read-only file-system then it is safe.
-	// Older copies of snapd were not mounting squashfs as read only.
-	if fsData.Type == SquashfsMagic {
-		return true, nil
-	}
-	return false, nil
-}
-
-// IsSnapdCreatedPrivateTmpfs returns true if a directory is a tmpfs mounted by snapd.
-//
-// The function inspects the directory (represented as both an open file
-// descriptor and the absolute path) and a list of changes that were applied to
-// the mount namespace. A directory is trusted if it is a tmpfs that was
-// mounted by snap-confine or snapd-update-ns. Note that sub-directories of a
-// trusted tmpfs are not considered trusted by this function.
-func IsSnapdCreatedPrivateTmpfs(dirFd int, dirName string, changes []*Change) (bool, error) {
-	var fsData syscall.Statfs_t
-	if err := sysFstatfs(dirFd, &fsData); err != nil {
-		return false, fmt.Errorf("cannot fstatfs %q: %s", dirName, err)
-	}
-	// If something is not a tmpfs it cannot be the trusted tmpfs we are looking for.
-	if fsData.Type != TmpfsMagic {
-		return false, nil
-	}
-	// Any of the past changes that mounted a tmpfs exactly at the directory we
-	// are inspecting is considered as trusted. This is conservative because it
-	// doesn't trust sub-directories of a trusted tmpfs. This approach is
-	// sufficient for the intended use.
-	//
-	// The algorithm goes over all the changes in reverse and picks up the
-	// first tmpfs mount or unmount action that matches the directory name.
-	// The set of constraints in snap-update-ns and snapd prevent from mounting
-	// over an existing mount point so we don't need to consider e.g. a bind
-	// mount shadowing an active tmpfs.
-	for i := len(changes) - 1; i >= 0; i-- {
-		change := changes[i]
-		if change.Entry.Type == "tmpfs" && change.Entry.Dir == dirName {
-			return change.Action == Mount, nil
-		}
-	}
-	// TODO: As a special exception, assume that a tmpfs over /var/lib is
-	// trusted. This tmpfs is created by snap-confine as a "quirk" to support
-	// a particular behavior of LXD.  Once the quirk is migrated to a mount
-	// profile (or removed entirely if no longer necessary) the following code
-	// fragment can go away.
-	return dirName == "/var/lib", nil
-}
-
 // ReadOnlyFsError is an error encapsulating encountered EROFS.
 type ReadOnlyFsError struct {
 	Path string
@@ -139,27 +78,12 @@ func (e *ReadOnlyFsError) Error() string {
 	return fmt.Sprintf("cannot operate on read-only filesystem at %s", e.Path)
 }
 
-// Secure is a helper for making filesystem operations free from certain kinds of attacks.
-type Secure struct{}
-
-// CheckTrespassing inspects if a filesystem operation on the given path
-// segment would trespass on the host.
-//
-// The idea of trespassing is so that we do not attempt to create a
-// directory, a file or a symlink in a directory that is usually beyond
-// control of snap applications. We are perfectly comfortable with creating
-// things in $SNAP_DATA or in /tmp but we don't want to do so in /etc or in
-// other sensitive places.
-func (sec *Secure) CheckTrespassing(dirFd int, dirName string, name string) error {
-	return nil
-}
-
 // OpenPath creates a path file descriptor for the given
 // path, making sure no components are symbolic links.
 //
 // The file descriptor is opened using the O_PATH, O_NOFOLLOW,
 // and O_CLOEXEC flags.
-func (sec *Secure) OpenPath(path string) (int, error) {
+func OpenPath(path string) (int, error) {
 	iter, err := strutil.NewPathIterator(path)
 	if err != nil {
 		return -1, fmt.Errorf("cannot open path: %s", err)
@@ -204,10 +128,10 @@ func (sec *Secure) OpenPath(path string) (int, error) {
 }
 
 // MkPrefix creates all the missing directories in a given base path and
-// returns the file descriptor to the leaf directory. This function is a base
-// for secure variants of mkdir, touch and symlink. None of the traversed
-// directories can be symbolic links.
-func (sec *Secure) MkPrefix(base string, perm os.FileMode, uid sys.UserID, gid sys.GroupID) (int, error) {
+// returns the file descriptor to the leaf directory as well as the restricted
+// flag. This function is a base for secure variants of mkdir, touch and
+// symlink. None of the traversed directories can be symbolic links.
+func MkPrefix(base string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, rs *Restrictions) (int, error) {
 	iter, err := strutil.NewPathIterator(base)
 	if err != nil {
 		// TODO: Reword the error and adjust the tests.
@@ -220,6 +144,10 @@ func (sec *Secure) MkPrefix(base string, perm os.FileMode, uid sys.UserID, gid s
 
 	const openFlags = syscall.O_NOFOLLOW | syscall.O_CLOEXEC | syscall.O_DIRECTORY
 	// Open the root directory and start there.
+	//
+	// We don't have to check for possible trespassing on / here because we are
+	// going to check for it in sec.MkDir call below which verifies that
+	// trespassing restrictions are not violated.
 	fd, err := sysOpen("/", openFlags, 0)
 	if err != nil {
 		return -1, fmt.Errorf("cannot open root directory: %v", err)
@@ -228,7 +156,7 @@ func (sec *Secure) MkPrefix(base string, perm os.FileMode, uid sys.UserID, gid s
 		// Keep closing the previous descriptor as we go, so that we have the
 		// last one handy from the MkDir below.
 		defer sysClose(fd)
-		fd, err = sec.MkDir(fd, iter.CurrentBase(), iter.CurrentCleanName(), perm, uid, gid)
+		fd, err = MkDir(fd, iter.CurrentBase(), iter.CurrentCleanName(), perm, uid, gid, rs)
 		if err != nil {
 			return -1, err
 		}
@@ -243,9 +171,12 @@ func (sec *Secure) MkPrefix(base string, perm os.FileMode, uid sys.UserID, gid s
 // convenience). This function is meant to be used to construct subsequent
 // elements of some path. The return value contains the newly created file
 // descriptor for the new directory or -1 on error.
-func (sec *Secure) MkDir(dirFd int, dirName string, name string, perm os.FileMode, uid sys.UserID, gid sys.GroupID) (int, error) {
-	made := true
+func MkDir(dirFd int, dirName string, name string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, rs *Restrictions) (int, error) {
+	if err := rs.Check(dirFd, dirName); err != nil {
+		return -1, err
+	}
 
+	made := true
 	const openFlags = syscall.O_NOFOLLOW | syscall.O_CLOEXEC | syscall.O_DIRECTORY
 
 	if err := sysMkdirat(dirFd, name, uint32(perm.Perm())); err != nil {
@@ -273,6 +204,16 @@ func (sec *Secure) MkDir(dirFd int, dirName string, name string, perm os.FileMod
 			sysClose(newFd)
 			return -1, fmt.Errorf("cannot chown directory %q to %d.%d: %v", filepath.Join(dirName, name), uid, gid, err)
 		}
+		// As soon as we find a place that is safe to write we can switch off
+		// the restricted mode (and thus any subsequent checks). This is
+		// because we only allow "writing" to read-only filesystems where
+		// writes fail with EROFS or to a tmpfs that snapd has privately
+		// mounted inside the per-snap mount namespace. As soon as we start
+		// walking over such tmpfs any subsequent children are either read-
+		// only bind mounts from $SNAP, other tmpfs'es  (e.g. one explicitly
+		// constructed for a layout) or writable places that are bind-mounted
+		// from $SNAP_DATA or similar.
+		rs.Lift()
 	}
 	return newFd, err
 }
@@ -280,12 +221,15 @@ func (sec *Secure) MkDir(dirFd int, dirName string, name string, perm os.FileMod
 // MkFile creates a file with a given name.
 //
 // The directory is represented with a file descriptor and its name (for
-// convenience). This function is meant to be used to create the leaf file as a
-// preparation for a mount point. Existing files are reused without errors.
+// convenience). This function is meant to be used to create the leaf file as
+// a preparation for a mount point. Existing files are reused without errors.
 // Newly created files have the specified mode and ownership.
-func (sec *Secure) MkFile(dirFd int, dirName string, name string, perm os.FileMode, uid sys.UserID, gid sys.GroupID) error {
-	made := true
+func MkFile(dirFd int, dirName string, name string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, rs *Restrictions) error {
+	if err := rs.Check(dirFd, dirName); err != nil {
+		return err
+	}
 
+	made := true
 	// NOTE: Tests don't show O_RDONLY as has a value of 0 and is not
 	// translated to textual form. It is added here for explicitness.
 	const openFlags = syscall.O_NOFOLLOW | syscall.O_CLOEXEC | syscall.O_RDONLY
@@ -330,9 +274,12 @@ func (sec *Secure) MkFile(dirFd int, dirName string, name string, perm os.FileMo
 // The directory is represented with a file descriptor and its name (for
 // convenience). This function is meant to be used to create the leaf symlink.
 // Existing and identical symlinks are reused without errors.
-func (sec *Secure) MkSymlink(dirFd int, dirName string, name string, oldname string) error {
+func MkSymlink(dirFd int, dirName string, name string, oldname string, rs *Restrictions) error {
+	if err := rs.Check(dirFd, dirName); err != nil {
+		return err
+	}
+
 	// Create the final path segment as a symlink.
-	// TODO: don't write links outside of tmpfs or $SNAP_{,USER_}{DATA,COMMON}
 	if err := sysSymlinkat(oldname, dirFd, name); err != nil {
 		switch err {
 		case syscall.EEXIST:
@@ -387,7 +334,7 @@ func (sec *Secure) MkSymlink(dirFd int, dirName string, name string, oldname str
 // The uid and gid are used for the fchown(2) system call which is performed
 // after each segment is created and opened. The special value -1 may be used
 // to request that ownership is not changed.
-func (sec *Secure) MkdirAll(path string, perm os.FileMode, uid sys.UserID, gid sys.GroupID) error {
+func MkdirAll(path string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, rs *Restrictions) error {
 	if path != filepath.Clean(path) {
 		// TODO: Reword the error and adjust the tests.
 		return fmt.Errorf("cannot split unclean path %q", path)
@@ -401,7 +348,7 @@ func (sec *Secure) MkdirAll(path string, perm os.FileMode, uid sys.UserID, gid s
 	base = filepath.Clean(base) // Needed to chomp the trailing slash.
 
 	// Create the prefix.
-	dirFd, err := sec.MkPrefix(base, perm, uid, gid)
+	dirFd, err := MkPrefix(base, perm, uid, gid, rs)
 	if err != nil {
 		return err
 	}
@@ -409,7 +356,7 @@ func (sec *Secure) MkdirAll(path string, perm os.FileMode, uid sys.UserID, gid s
 
 	if name != "" {
 		// Create the leaf as a directory.
-		leafFd, err := sec.MkDir(dirFd, base, name, perm, uid, gid)
+		leafFd, err := MkDir(dirFd, base, name, perm, uid, gid, rs)
 		if err != nil {
 			return err
 		}
@@ -424,7 +371,7 @@ func (sec *Secure) MkdirAll(path string, perm os.FileMode, uid sys.UserID, gid s
 // This function is like MkdirAll but it creates an empty file instead of
 // a directory for the final path component. Each created directory component
 // is chowned to the desired user and group.
-func (sec *Secure) MkfileAll(path string, perm os.FileMode, uid sys.UserID, gid sys.GroupID) error {
+func MkfileAll(path string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, rs *Restrictions) error {
 	if path != filepath.Clean(path) {
 		// TODO: Reword the error and adjust the tests.
 		return fmt.Errorf("cannot split unclean path %q", path)
@@ -442,7 +389,7 @@ func (sec *Secure) MkfileAll(path string, perm os.FileMode, uid sys.UserID, gid
 	base = filepath.Clean(base) // Needed to chomp the trailing slash.
 
 	// Create the prefix.
-	dirFd, err := sec.MkPrefix(base, perm, uid, gid)
+	dirFd, err := MkPrefix(base, perm, uid, gid, rs)
 	if err != nil {
 		return err
 	}
@@ -450,13 +397,13 @@ func (sec *Secure) MkfileAll(path string, perm os.FileMode, uid sys.UserID, gid
 
 	if name != "" {
 		// Create the leaf as a file.
-		err = sec.MkFile(dirFd, base, name, perm, uid, gid)
+		err = MkFile(dirFd, base, name, perm, uid, gid, rs)
 	}
 	return err
 }
 
 // MksymlinkAll is a secure implementation of "ln -s".
-func (sec *Secure) MksymlinkAll(path string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, oldname string) error {
+func MksymlinkAll(path string, perm os.FileMode, uid sys.UserID, gid sys.GroupID, oldname string, rs *Restrictions) error {
 	if path != filepath.Clean(path) {
 		// TODO: Reword the error and adjust the tests.
 		return fmt.Errorf("cannot split unclean path %q", path)
@@ -478,7 +425,7 @@ func (sec *Secure) MksymlinkAll(path string, perm os.FileMode, uid sys.UserID, g
 	base = filepath.Clean(base) // Needed to chomp the trailing slash.
 
 	// Create the prefix.
-	dirFd, err := sec.MkPrefix(base, perm, uid, gid)
+	dirFd, err := MkPrefix(base, perm, uid, gid, rs)
 	if err != nil {
 		return err
 	}
@@ -486,7 +433,7 @@ func (sec *Secure) MksymlinkAll(path string, perm os.FileMode, uid sys.UserID, g
 
 	if name != "" {
 		// Create the leaf as a symlink.
-		err = sec.MkSymlink(dirFd, base, name, oldname)
+		err = MkSymlink(dirFd, base, name, oldname, rs)
 	}
 	return err
 }
@@ -621,10 +568,10 @@ type FatalError struct {
 // In the event of a failure the undo plan is executed and an error is
 // returned. If the undo plan fails the function returns a FatalError as it
 // cannot fix the system from an inconsistent state.
-func execWritableMimic(plan []*Change, sec *Secure) ([]*Change, error) {
+func execWritableMimic(plan []*Change, as *Assumptions) ([]*Change, error) {
 	undoChanges := make([]*Change, 0, len(plan)-2)
 	for i, change := range plan {
-		if _, err := changePerform(change, sec); err != nil {
+		if _, err := changePerform(change, as); err != nil {
 			// Drat, we failed! Let's undo everything according to our own undo
 			// plan, by following it in reverse order.
 
@@ -647,7 +594,7 @@ func execWritableMimic(plan []*Change, sec *Secure) ([]*Change, error) {
 				if recoveryUndoChange.Entry.OptBool("rbind") {
 					recoveryUndoChange.Entry.Options = append(recoveryUndoChange.Entry.Options, osutil.XSnapdDetach())
 				}
-				if _, err2 := changePerform(recoveryUndoChange, sec); err2 != nil {
+				if _, err2 := changePerform(recoveryUndoChange, as); err2 != nil {
 					// Drat, we failed when trying to recover from an error.
 					// We cannot do anything at this stage.
 					return nil, &FatalError{error: fmt.Errorf("cannot undo change %q while recovering from earlier error %v: %v", recoveryUndoChange, err, err2)}
@@ -695,12 +642,12 @@ func execWritableMimic(plan []*Change, sec *Secure) ([]*Change, error) {
 	return undoChanges, nil
 }
 
-func createWritableMimic(dir, neededBy string, sec *Secure) ([]*Change, error) {
+func createWritableMimic(dir, neededBy string, as *Assumptions) ([]*Change, error) {
 	plan, err := planWritableMimic(dir, neededBy)
 	if err != nil {
 		return nil, err
 	}
-	changes, err := execWritableMimic(plan, sec)
+	changes, err := execWritableMimic(plan, as)
 	if err != nil {
 		return nil, err
 	}
