@@ -29,8 +29,8 @@ type cmdBooted struct{}
 
 func init() {
 	cmd := addCommand("booted",
-		"internal",
-		"internal",
+		"Internal",
+		"The booted command is only retained for backwards compatibility.",
 		func() flags.Commander {
 			return &cmdBooted{}
 		}, nil, nil)
