@@ -48,18 +48,20 @@ core device images.
 		func() flags.Commander {
 			return &cmdPrepareImage{}
 		}, map[string]string{
-			"extra-snaps": "Extra snaps to be installed",
-			"channel":     "The channel to use",
+			// TRANSLATORS: This should not start with a lowercase letter.
+			"extra-snaps": i18n.G("Extra snaps to be installed"),
+			// TRANSLATORS: This should not start with a lowercase letter.
+			"channel": i18n.G("The channel to use"),
 		}, []argDesc{
 			{
-				// TRANSLATORS: This needs to be wrapped in <>s.
+				// TRANSLATORS: This needs to begin with < and end with >
 				name: i18n.G("<model-assertion>"),
-				// TRANSLATORS: This should probably not start with a lowercase letter.
+				// TRANSLATORS: This should not start with a lowercase letter.
 				desc: i18n.G("The model assertion name"),
 			}, {
-				// TRANSLATORS: This needs to be wrapped in <>s.
+				// TRANSLATORS: This needs to begin with < and end with >
 				name: i18n.G("<root-dir>"),
-				// TRANSLATORS: This should probably not start with a lowercase letter.
+				// TRANSLATORS: This should not start with a lowercase letter.
 				desc: i18n.G("The output directory"),
 			},
 		})
