@@ -27,12 +27,17 @@ import (
 
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/overlord/auth"
+	"github.com/snapcore/snapd/selinux"
 	"github.com/snapcore/snapd/store"
 )
 
 var RunMain = run
 
 var (
+	Client = mkClient
+
+	FirstNonOptionIsRun = firstNonOptionIsRun
+
 	CreateUserDataDirs = createUserDataDirs
 	ResolveApp         = resolveApp
 	IsReexeced         = isReexeced
@@ -56,6 +61,21 @@ var (
 	ShortPublisher       = shortPublisher
 
 	ReadRpc = readRpc
+
+	WriteWarningTimestamp = writeWarningTimestamp
+	MaybePresentWarnings  = maybePresentWarnings
+
+	LongSnapDescription     = longSnapDescription
+	SnapUsage               = snapUsage
+	SnapHelpCategoriesIntro = snapHelpCategoriesIntro
+	SnapHelpAllFooter       = snapHelpAllFooter
+	SnapHelpFooter          = snapHelpFooter
+	HelpCategories          = helpCategories
+
+	LintArg  = lintArg
+	LintDesc = lintDesc
+
+	FixupArg = fixupArg
 )
 
 func MockPollTime(d time.Duration) (restore func()) {
@@ -144,19 +164,19 @@ func AssertTypeNameCompletion(match string) []flags.Completion {
 	return assertTypeName("").Complete(match)
 }
 
-func MockIsTTY(t bool) (restore func()) {
-	oldIsTTY := isTTY
-	isTTY = t
+func MockIsStdoutTTY(t bool) (restore func()) {
+	oldIsStdoutTTY := isStdoutTTY
+	isStdoutTTY = t
 	return func() {
-		isTTY = oldIsTTY
+		isStdoutTTY = oldIsStdoutTTY
 	}
 }
 
-func MockIsTerminal(t bool) (restore func()) {
-	oldIsTerminal := isTerminal
-	isTerminal = func() bool { return t }
+func MockIsStdinTTY(t bool) (restore func()) {
+	oldIsStdinTTY := isStdinTTY
+	isStdinTTY = t
 	return func() {
-		isTerminal = oldIsTerminal
+		isStdinTTY = oldIsStdinTTY
 	}
 }
 
@@ -185,7 +205,9 @@ func MockWaitConfTimeout(d time.Duration) (restore func()) {
 }
 
 func Wait(cli *client.Client, id string) (*client.Change, error) {
-	return waitMixin{}.wait(cli, id)
+	wmx := waitMixin{}
+	wmx.client = cli
+	return wmx.wait(id)
 }
 
 func ColorMixin(cmode, umode string) colorMixin {
@@ -195,3 +217,27 @@ func ColorMixin(cmode, umode string) colorMixin {
 func CmdAdviseSnap() *cmdAdviseSnap {
 	return &cmdAdviseSnap{}
 }
+
+func MockSELinuxIsEnabled(isEnabled func() (bool, error)) (restore func()) {
+	old := selinuxIsEnabled
+	selinuxIsEnabled = isEnabled
+	return func() {
+		selinuxIsEnabled = old
+	}
+}
+
+func MockSELinuxVerifyPathContext(verifypathcon func(string) (bool, error)) (restore func()) {
+	old := selinuxVerifyPathContext
+	selinuxVerifyPathContext = verifypathcon
+	return func() {
+		selinuxVerifyPathContext = old
+	}
+}
+
+func MockSELinuxRestoreContext(restorecon func(string, selinux.RestoreMode) error) (restore func()) {
+	old := selinuxRestoreContext
+	selinuxRestoreContext = restorecon
+	return func() {
+		selinuxRestoreContext = old
+	}
+}
