@@ -38,3 +38,11 @@ var (
 func (tsto *ToolingStore) User() *auth.UserState {
 	return tsto.user
 }
+
+func (ls *localInfos) NameToPath() map[string]string {
+	return ls.nameToPath
+}
+
+func ToolingAuthContext() auth.AuthContext {
+	return toolingAuthContext{}
+}
