@@ -48,79 +48,91 @@ const bluezPermanentSlotAppArmor = `
 # Description: Allow operating as the bluez service. This gives privileged
 # access to the system.
 
-  network bluetooth,
+network bluetooth,
 
-  capability net_admin,
-  capability net_bind_service,
+capability net_admin,
+capability net_bind_service,
 
-  # libudev
-  network netlink raw,
+# libudev
+network netlink raw,
 
-  # File accesses
-  /sys/bus/usb/drivers/btusb/     r,
-  /sys/bus/usb/drivers/btusb/**   r,
-  /sys/class/bluetooth/           r,
-  /sys/devices/**/bluetooth/      rw,
-  /sys/devices/**/bluetooth/**    rw,
-  /sys/devices/**/id/chassis_type r,
+# File accesses
+/sys/bus/usb/drivers/btusb/     r,
+/sys/bus/usb/drivers/btusb/**   r,
+/sys/class/bluetooth/           r,
+/sys/devices/**/bluetooth/      rw,
+/sys/devices/**/bluetooth/**    rw,
+/sys/devices/**/id/chassis_type r,
 
-  # TODO: use snappy hardware assignment for this once LP: #1498917 is fixed
-  /dev/rfkill rw,
+# TODO: use snappy hardware assignment for this once LP: #1498917 is fixed
+/dev/rfkill rw,
 
-  # DBus accesses
-  #include <abstractions/dbus-strict>
-  dbus (send)
-     bus=system
-     path=/org/freedesktop/DBus
-     interface=org.freedesktop.DBus
-     member={Request,Release}Name
-     peer=(name=org.freedesktop.DBus, label=unconfined),
+# DBus accesses
+#include <abstractions/dbus-strict>
+dbus (send)
+   bus=system
+   path=/org/freedesktop/DBus
+   interface=org.freedesktop.DBus
+   member={Request,Release}Name
+   peer=(name=org.freedesktop.DBus, label=unconfined),
 
-  dbus (send)
+dbus (send)
+  bus=system
+  path=/org/freedesktop/*
+  interface=org.freedesktop.DBus.Properties
+  peer=(label=unconfined),
+
+# Allow binding the service to the requested connection name
+dbus (bind)
     bus=system
-    path=/org/freedesktop/*
+    name="org.bluez",
+
+# Allow binding the service to the requested connection name
+dbus (bind)
+    bus=system
+    name="org.bluez.obex",
+
+# Allow traffic to/from our interface with any method for unconfined clients
+# to talk to our bluez services. For the org.bluez interface we don't specify
+# an Object Path since according to the bluez specification these can be
+# anything (https://git.kernel.org/pub/scm/bluetooth/bluez.git/tree/doc).
+dbus (receive, send)
+    bus=system
+    interface=org.bluez.*
+    peer=(label=unconfined),
+dbus (receive, send)
+    bus=system
+    path=/org/bluez{,/**}
+    interface=org.freedesktop.DBus.*
+    peer=(label=unconfined),
+
+# Allow traffic to/from org.freedesktop.DBus for bluez service. This rule is
+# not snap-specific and grants privileged access to the org.freedesktop.DBus
+# on the system bus.
+dbus (receive, send)
+    bus=system
+    path=/
+    interface=org.freedesktop.DBus.*
+    peer=(label=unconfined),
+
+# Allow access to hostname system service
+dbus (receive, send)
+    bus=system
+    path=/org/freedesktop/hostname1
     interface=org.freedesktop.DBus.Properties
     peer=(label=unconfined),
 
-  # Allow binding the service to the requested connection name
-  dbus (bind)
-      bus=system
-      name="org.bluez",
-
-  # Allow binding the service to the requested connection name
-  dbus (bind)
-      bus=system
-      name="org.bluez.obex",
-
-  # Allow traffic to/from our interface with any method for unconfined clients
-  # to talk to our bluez services. For the org.bluez interface we don't specify
-  # an Object Path since according to the bluez specification these can be
-  # anything (https://git.kernel.org/pub/scm/bluetooth/bluez.git/tree/doc).
-  dbus (receive, send)
-      bus=system
-      interface=org.bluez.*
-      peer=(label=unconfined),
-  dbus (receive, send)
-      bus=system
-      path=/org/bluez{,/**}
-      interface=org.freedesktop.DBus.*
-      peer=(label=unconfined),
-
-  # Allow traffic to/from org.freedesktop.DBus for bluez service. This rule is
-  # not snap-specific and grants privileged access to the org.freedesktop.DBus
-  # on the system bus.
-  dbus (receive, send)
-      bus=system
-      path=/
-      interface=org.freedesktop.DBus.*
-      peer=(label=unconfined),
-
-  # Allow access to hostname system service
-  dbus (receive, send)
-      bus=system
-      path=/org/freedesktop/hostname1
-      interface=org.freedesktop.DBus.Properties
-      peer=(label=unconfined),
+# do not use peer=(label=unconfined) here since this is DBus activated
+dbus (send)
+    bus=system
+    path=/org/freedesktop/hostname1
+    interface=org.freedesktop.DBus.Properties
+    member="Get{,All}",
+dbus (send)
+    bus=system
+    path=/org/freedesktop/hostname1
+    interface=org.freedesktop.DBus.Introspectable
+    member=Introspect,
 `
 
 const bluezConnectedSlotAppArmor = `
