@@ -114,8 +114,7 @@ func (s *TimeControlInterfaceSuite) TestStaticInfo(c *C) {
 }
 
 func (s *TimeControlInterfaceSuite) TestAutoConnect(c *C) {
-	// FIXME: fix AutoConnect methods to use ConnectedPlug/Slot
-	c.Assert(s.iface.AutoConnect(&interfaces.Plug{PlugInfo: s.plugInfo}, &interfaces.Slot{SlotInfo: s.slotInfo}), Equals, true)
+	c.Assert(s.iface.AutoConnect(s.plugInfo, s.slotInfo), Equals, true)
 }
 
 func (s *TimeControlInterfaceSuite) TestInterfaces(c *C) {
