@@ -25,7 +25,6 @@ import (
 	. "gopkg.in/check.v1"
 
 	. "github.com/snapcore/snapd/interfaces"
-	"github.com/snapcore/snapd/interfaces/hotplug"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
@@ -36,6 +35,7 @@ type RepositorySuite struct {
 	testutil.BaseTest
 	iface     Interface
 	plug      *snap.PlugInfo
+	plugSelf  *snap.PlugInfo
 	slot      *snap.SlotInfo
 	emptyRepo *Repository
 	// Repository pre-populated with s.iface
@@ -53,11 +53,7 @@ var _ = Suite(&RepositorySuite{
 	},
 })
 
-func (s *RepositorySuite) SetUpTest(c *C) {
-	s.BaseTest.SetUpTest(c)
-	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
-
-	consumer := snaptest.MockInfo(c, `
+const consumerYaml = `
 name: consumer
 version: 0
 apps:
@@ -69,9 +65,9 @@ plugs:
         interface: interface
         label: label
         attr: value
-`, nil)
-	s.plug = consumer.Plugs["plug"]
-	producer := snaptest.MockInfo(c, `
+`
+
+const producerYaml = `
 name: producer
 version: 0
 apps:
@@ -83,9 +79,21 @@ slots:
         interface: interface
         label: label
         attr: value
-`, nil)
-	s.slot = producer.Slots["slot"]
+plugs:
+    self:
+        interface: interface
+        label: label
+`
 
+func (s *RepositorySuite) SetUpTest(c *C) {
+	s.BaseTest.SetUpTest(c)
+	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
+
+	consumer := snaptest.MockInfo(c, consumerYaml, nil)
+	s.plug = consumer.Plugs["plug"]
+	producer := snaptest.MockInfo(c, producerYaml, nil)
+	s.slot = producer.Slots["slot"]
+	s.plugSelf = producer.Plugs["self"]
 	// NOTE: Each of the snaps below have one slot so that they can be picked
 	// up by the repository. Some tests rename the "slot" slot as appropriate.
 	s.ubuntuCoreSnap = snaptest.MockInfo(c, `
@@ -96,6 +104,9 @@ slots:
     slot:
         interface: interface
 `, nil)
+	// NOTE: The core snap has a slot so that it shows up in the
+	// repository. The repository doesn't record snaps unless they
+	// have at least one interface.
 	s.coreSnap = snaptest.MockInfo(c, `
 name: core
 version: 0
@@ -123,11 +134,22 @@ func (s *RepositorySuite) TearDownTest(c *C) {
 	s.BaseTest.TearDownTest(c)
 }
 
-func addPlugsSlots(c *C, repo *Repository, yamls ...string) []*snap.Info {
-	result := make([]*snap.Info, len(yamls))
-	for i, yaml := range yamls {
-		info := snaptest.MockInfo(c, yaml, nil)
-		result[i] = info
+type instanceNameAndYaml struct {
+	Name string
+	Yaml string
+}
+
+func addPlugsSlotsFromInstances(c *C, repo *Repository, iys []instanceNameAndYaml) []*snap.Info {
+	result := make([]*snap.Info, 0, len(iys))
+	for _, iy := range iys {
+		info := snaptest.MockInfo(c, iy.Yaml, nil)
+		if iy.Name != "" {
+			instanceName := iy.Name
+			c.Assert(snap.ValidateInstanceName(instanceName), IsNil)
+			_, info.InstanceKey = snap.SplitInstanceName(instanceName)
+		}
+
+		result = append(result, info)
 		for _, plugInfo := range info.Plugs {
 			err := repo.AddPlug(plugInfo)
 			c.Assert(err, IsNil)
@@ -300,6 +322,23 @@ func (s *RepositorySuite) TestAddPlugFailsWithUnknownInterface(c *C) {
 	c.Assert(s.emptyRepo.AllPlugs(""), HasLen, 0)
 }
 
+func (s *RepositorySuite) TestAddPlugParallelInstance(c *C) {
+	c.Assert(s.testRepo.AllPlugs(""), HasLen, 0)
+
+	err := s.testRepo.AddPlug(s.plug)
+	c.Assert(err, IsNil)
+	c.Assert(s.testRepo.AllPlugs(""), HasLen, 1)
+
+	consumer := snaptest.MockInfo(c, consumerYaml, nil)
+	consumer.InstanceKey = "instance"
+	err = s.testRepo.AddPlug(consumer.Plugs["plug"])
+	c.Assert(err, IsNil)
+	c.Assert(s.testRepo.AllPlugs(""), HasLen, 2)
+
+	c.Assert(s.testRepo.Plug(s.plug.Snap.InstanceName(), s.plug.Name), DeepEquals, s.plug)
+	c.Assert(s.testRepo.Plug(consumer.InstanceName(), "plug"), DeepEquals, consumer.Plugs["plug"])
+}
+
 // Tests for Repository.Plug()
 
 func (s *RepositorySuite) TestPlug(c *C) {
@@ -310,28 +349,42 @@ func (s *RepositorySuite) TestPlug(c *C) {
 }
 
 func (s *RepositorySuite) TestPlugSearch(c *C) {
-	addPlugsSlots(c, s.testRepo, `
-name: x
+	addPlugsSlotsFromInstances(c, s.testRepo, []instanceNameAndYaml{
+		{Name: "xx", Yaml: `
+name: xx
 version: 0
 plugs:
     a: interface
     b: interface
     c: interface
-`, `
-name: y
+`},
+		{Name: "yy", Yaml: `
+name: yy
 version: 0
 plugs:
     a: interface
     b: interface
     c: interface
-`)
+`},
+		{Name: "zz_instance", Yaml: `
+name: zz
+version: 0
+plugs:
+    a: interface
+    b: interface
+    c: interface
+`},
+	})
 	// Plug() correctly finds plugs
-	c.Assert(s.testRepo.Plug("x", "a"), Not(IsNil))
-	c.Assert(s.testRepo.Plug("x", "b"), Not(IsNil))
-	c.Assert(s.testRepo.Plug("x", "c"), Not(IsNil))
-	c.Assert(s.testRepo.Plug("y", "a"), Not(IsNil))
-	c.Assert(s.testRepo.Plug("y", "b"), Not(IsNil))
-	c.Assert(s.testRepo.Plug("y", "c"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("xx", "a"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("xx", "b"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("xx", "c"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("yy", "a"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("yy", "b"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("yy", "c"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("zz_instance", "a"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("zz_instance", "b"), Not(IsNil))
+	c.Assert(s.testRepo.Plug("zz_instance", "c"), Not(IsNil))
 }
 
 // Tests for Repository.RemovePlug()
@@ -355,7 +408,7 @@ func (s *RepositorySuite) TestRemovePlugFailsWhenPlugIsConnected(c *C) {
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	// Removing a plug used by a slot returns an appropriate error
 	err = s.testRepo.RemovePlug(s.plug.Snap.InstanceName(), s.plug.Name)
@@ -368,25 +421,40 @@ func (s *RepositorySuite) TestRemovePlugFailsWhenPlugIsConnected(c *C) {
 // Tests for Repository.AllPlugs()
 
 func (s *RepositorySuite) TestAllPlugsWithoutInterfaceName(c *C) {
-	snaps := addPlugsSlots(c, s.testRepo, `
+	snaps := addPlugsSlotsFromInstances(c, s.testRepo, []instanceNameAndYaml{
+		{Name: "snap-a", Yaml: `
 name: snap-a
 version: 0
 plugs:
     name-a: interface
-`, `
+`},
+		{Name: "snap-b", Yaml: `
 name: snap-b
 version: 0
 plugs:
     name-a: interface
     name-b: interface
     name-c: interface
-`)
+`},
+		{Name: "snap-b_instance", Yaml: `
+name: snap-b
+version: 0
+plugs:
+    name-a: interface
+    name-b: interface
+    name-c: interface
+`},
+	})
+	c.Assert(snaps, HasLen, 3)
 	// The result is sorted by snap and name
 	c.Assert(s.testRepo.AllPlugs(""), DeepEquals, []*snap.PlugInfo{
 		snaps[0].Plugs["name-a"],
 		snaps[1].Plugs["name-a"],
 		snaps[1].Plugs["name-b"],
 		snaps[1].Plugs["name-c"],
+		snaps[2].Plugs["name-a"],
+		snaps[2].Plugs["name-b"],
+		snaps[2].Plugs["name-c"],
 	})
 }
 
@@ -394,46 +462,79 @@ func (s *RepositorySuite) TestAllPlugsWithInterfaceName(c *C) {
 	// Add another interface so that we can look for it
 	err := s.testRepo.AddInterface(&ifacetest.TestInterface{InterfaceName: "other-interface"})
 	c.Assert(err, IsNil)
-	snaps := addPlugsSlots(c, s.testRepo, `
+	snaps := addPlugsSlotsFromInstances(c, s.testRepo, []instanceNameAndYaml{
+		{Name: "snap-a", Yaml: `
 name: snap-a
 version: 0
 plugs:
     name-a: interface
-`, `
+`},
+		{Name: "snap-b", Yaml: `
 name: snap-b
 version: 0
 plugs:
     name-a: interface
     name-b: other-interface
     name-c: interface
-`)
-	c.Assert(s.testRepo.AllPlugs("other-interface"), DeepEquals, []*snap.PlugInfo{snaps[1].Plugs["name-b"]})
+`},
+		{Name: "snap-b_instance", Yaml: `
+name: snap-b
+version: 0
+plugs:
+    name-a: interface
+    name-b: other-interface
+    name-c: interface
+`},
+	})
+	c.Assert(snaps, HasLen, 3)
+	c.Assert(s.testRepo.AllPlugs("other-interface"), DeepEquals, []*snap.PlugInfo{
+		snaps[1].Plugs["name-b"],
+		snaps[2].Plugs["name-b"],
+	})
 }
 
 // Tests for Repository.Plugs()
 
 func (s *RepositorySuite) TestPlugs(c *C) {
-	snaps := addPlugsSlots(c, s.testRepo, `
+	snaps := addPlugsSlotsFromInstances(c, s.testRepo, []instanceNameAndYaml{
+		{Name: "snap-a", Yaml: `
 name: snap-a
 version: 0
 plugs:
     name-a: interface
-`, `
+`},
+		{Name: "snap-b", Yaml: `
 name: snap-b
 version: 0
 plugs:
     name-a: interface
     name-b: interface
     name-c: interface
-`)
+`},
+		{Name: "snap-b_instance", Yaml: `
+name: snap-b
+version: 0
+plugs:
+    name-a: interface
+    name-b: interface
+    name-c: interface
+`},
+	})
+	c.Assert(snaps, HasLen, 3)
 	// The result is sorted by snap and name
 	c.Assert(s.testRepo.Plugs("snap-b"), DeepEquals, []*snap.PlugInfo{
 		snaps[1].Plugs["name-a"],
 		snaps[1].Plugs["name-b"],
 		snaps[1].Plugs["name-c"],
 	})
+	c.Assert(s.testRepo.Plugs("snap-b_instance"), DeepEquals, []*snap.PlugInfo{
+		snaps[2].Plugs["name-a"],
+		snaps[2].Plugs["name-b"],
+		snaps[2].Plugs["name-c"],
+	})
 	// The result is empty if the snap is not known
 	c.Assert(s.testRepo.Plugs("snap-x"), HasLen, 0)
+	c.Assert(s.testRepo.Plugs("snap-b_other"), HasLen, 0)
 }
 
 // Tests for Repository.AllSlots()
@@ -441,45 +542,66 @@ plugs:
 func (s *RepositorySuite) TestAllSlots(c *C) {
 	err := s.testRepo.AddInterface(&ifacetest.TestInterface{InterfaceName: "other-interface"})
 	c.Assert(err, IsNil)
-	snaps := addPlugsSlots(c, s.testRepo, `
+	snaps := addPlugsSlotsFromInstances(c, s.testRepo, []instanceNameAndYaml{
+		{Name: "snap-a", Yaml: `
 name: snap-a
 version: 0
 slots:
     name-a: interface
     name-b: interface
-`, `
+`},
+		{Name: "snap-b", Yaml: `
 name: snap-b
 version: 0
 slots:
     name-a: other-interface
-`)
+`},
+		{Name: "snap-b_instance", Yaml: `
+name: snap-b
+version: 0
+slots:
+    name-a: other-interface
+`},
+	})
+	c.Assert(snaps, HasLen, 3)
 	// AllSlots("") returns all slots, sorted by snap and slot name
 	c.Assert(s.testRepo.AllSlots(""), DeepEquals, []*snap.SlotInfo{
 		snaps[0].Slots["name-a"],
 		snaps[0].Slots["name-b"],
 		snaps[1].Slots["name-a"],
+		snaps[2].Slots["name-a"],
 	})
 	// AllSlots("") returns all slots, sorted by snap and slot name
 	c.Assert(s.testRepo.AllSlots("other-interface"), DeepEquals, []*snap.SlotInfo{
 		snaps[1].Slots["name-a"],
+		snaps[2].Slots["name-a"],
 	})
 }
 
 // Tests for Repository.Slots()
 
 func (s *RepositorySuite) TestSlots(c *C) {
-	snaps := addPlugsSlots(c, s.testRepo, `
+	snaps := addPlugsSlotsFromInstances(c, s.testRepo, []instanceNameAndYaml{
+		{Name: "snap-a", Yaml: `
 name: snap-a
 version: 0
 slots:
     name-a: interface
     name-b: interface
-`, `
+`},
+		{Name: "snap-b", Yaml: `
 name: snap-b
 version: 0
 slots:
     name-a: interface
-`)
+`},
+		{Name: "snap-b_instance", Yaml: `
+name: snap-b
+version: 0
+slots:
+    name-a: interface
+`},
+	})
 	// Slots("snap-a") returns slots present in that snap
 	c.Assert(s.testRepo.Slots("snap-a"), DeepEquals, []*snap.SlotInfo{
 		snaps[0].Slots["name-a"],
@@ -489,8 +611,14 @@ slots:
 	c.Assert(s.testRepo.Slots("snap-b"), DeepEquals, []*snap.SlotInfo{
 		snaps[1].Slots["name-a"],
 	})
+	// Slots("snap-b_instance") returns slots present in that snap
+	c.Assert(s.testRepo.Slots("snap-b_instance"), DeepEquals, []*snap.SlotInfo{
+		snaps[2].Slots["name-a"],
+	})
 	// Slots("snap-c") returns no slots (because that snap doesn't exist)
 	c.Assert(s.testRepo.Slots("snap-c"), HasLen, 0)
+	// Slots("snap-b_other") returns no slots (the snap does not exist)
+	c.Assert(s.testRepo.Slots("snap-b_other"), HasLen, 0)
 	// Slots("") returns no slots
 	c.Assert(s.testRepo.Slots(""), HasLen, 0)
 }
@@ -575,6 +703,23 @@ func (s *RepositorySuite) TestAddSlotStoresCorrectData(c *C) {
 	c.Assert(slot, DeepEquals, s.slot)
 }
 
+func (s *RepositorySuite) TestAddSlotParallelInstance(c *C) {
+	c.Assert(s.testRepo.AllSlots(""), HasLen, 0)
+
+	err := s.testRepo.AddSlot(s.slot)
+	c.Assert(err, IsNil)
+	c.Assert(s.testRepo.AllSlots(""), HasLen, 1)
+
+	producer := snaptest.MockInfo(c, producerYaml, nil)
+	producer.InstanceKey = "instance"
+	err = s.testRepo.AddSlot(producer.Slots["slot"])
+	c.Assert(err, IsNil)
+	c.Assert(s.testRepo.AllSlots(""), HasLen, 2)
+
+	c.Assert(s.testRepo.Slot(s.slot.Snap.InstanceName(), s.slot.Name), DeepEquals, s.slot)
+	c.Assert(s.testRepo.Slot(producer.InstanceName(), "slot"), DeepEquals, producer.Slots["slot"])
+}
+
 // Tests for Repository.RemoveSlot()
 
 func (s *RepositorySuite) TestRemoveSlotSuccedsWhenSlotExistsAndDisconnected(c *C) {
@@ -601,7 +746,7 @@ func (s *RepositorySuite) TestRemoveSlotFailsWhenSlotIsConnected(c *C) {
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	// Removing a slot occupied by a plug returns an appropriate error
 	err = s.testRepo.RemoveSlot(s.slot.Snap.InstanceName(), s.slot.Name)
@@ -1081,7 +1226,7 @@ func (s *RepositorySuite) TestResolveDisconnectMatrixTypical(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err := s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err := s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	scenarios := []struct {
@@ -1163,7 +1308,7 @@ func (s *RepositorySuite) TestConnectFailsWhenPlugDoesNotExist(c *C) {
 	c.Assert(err, IsNil)
 	// Connecting an unknown plug returns an appropriate error
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, ErrorMatches, `cannot connect plug "plug" from snap "consumer": no such plug`)
 }
 
@@ -1172,7 +1317,7 @@ func (s *RepositorySuite) TestConnectFailsWhenSlotDoesNotExist(c *C) {
 	c.Assert(err, IsNil)
 	// Connecting to an unknown slot returns an error
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, ErrorMatches, `cannot connect slot "slot" from snap "producer": no such slot`)
 }
 
@@ -1182,7 +1327,7 @@ func (s *RepositorySuite) TestConnectSucceedsWhenIdenticalConnectExists(c *C) {
 	err = s.testRepo.AddSlot(s.slot)
 	c.Assert(err, IsNil)
 	connRef := NewConnRef(s.plug, s.slot)
-	conn, err := s.testRepo.Connect(connRef, nil, nil, nil)
+	conn, err := s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	c.Assert(conn, NotNil)
 	c.Assert(conn.Plug, NotNil)
@@ -1190,7 +1335,7 @@ func (s *RepositorySuite) TestConnectSucceedsWhenIdenticalConnectExists(c *C) {
 	c.Assert(conn.Plug.Name(), Equals, "plug")
 	c.Assert(conn.Slot.Name(), Equals, "slot")
 	// Connecting exactly the same thing twice succeeds without an error but does nothing.
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	// Only one connection is actually present.
 	c.Assert(s.testRepo.Interfaces(), DeepEquals, &Interfaces{
@@ -1215,7 +1360,7 @@ func (s *RepositorySuite) TestConnectFailsWhenSlotAndPlugAreIncompatible(c *C) {
 	c.Assert(err, IsNil)
 	// Connecting a plug to an incompatible slot fails with an appropriate error
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, ErrorMatches, `cannot connect plug "consumer:plug" \(interface "other-interface"\) to "producer:slot" \(interface "interface"\)`)
 }
 
@@ -1226,7 +1371,7 @@ func (s *RepositorySuite) TestConnectSucceeds(c *C) {
 	c.Assert(err, IsNil)
 	// Connecting a plug works okay
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 }
 
@@ -1270,9 +1415,9 @@ func (s *RepositorySuite) TestDisconnectFailsWhenNotConnected(c *C) {
 func (s *RepositorySuite) TestDisconnectSucceeds(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil)
+	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
-	_, err = s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil)
+	_, err = s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	err = s.testRepo.Disconnect(s.plug.Snap.InstanceName(), s.plug.Name, s.slot.Snap.InstanceName(), s.slot.Name)
 	c.Assert(err, IsNil)
@@ -1308,7 +1453,7 @@ func (s *RepositorySuite) TestConnectedFailsWithoutPlugOrSlot(c *C) {
 func (s *RepositorySuite) TestConnectedFindsConnections(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil)
+	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	conns, err := s.testRepo.Connected(s.plug.Snap.InstanceName(), s.plug.Name)
@@ -1329,7 +1474,7 @@ func (s *RepositorySuite) TestConnectedFindsCoreSnap(c *C) {
 	}
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(slot), IsNil)
-	_, err := s.testRepo.Connect(NewConnRef(s.plug, slot), nil, nil, nil)
+	_, err := s.testRepo.Connect(NewConnRef(s.plug, slot), nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	conns, err := s.testRepo.Connected("", s.slot.Name)
@@ -1341,7 +1486,7 @@ func (s *RepositorySuite) TestConnectedFindsCoreSnap(c *C) {
 func (s *RepositorySuite) TestConnections(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil)
+	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	conns, err := s.testRepo.Connections(s.plug.Snap.InstanceName())
@@ -1357,12 +1502,27 @@ func (s *RepositorySuite) TestConnections(c *C) {
 	c.Assert(conns, HasLen, 0)
 }
 
+func (s *RepositorySuite) TestConnectionsWithSelfConnected(c *C) {
+	c.Assert(s.testRepo.AddPlug(s.plugSelf), IsNil)
+	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
+	_, err := s.testRepo.Connect(NewConnRef(s.plugSelf, s.slot), nil, nil, nil, nil, nil)
+	c.Assert(err, IsNil)
+
+	conns, err := s.testRepo.Connections(s.plugSelf.Snap.InstanceName())
+	c.Assert(err, IsNil)
+	c.Check(conns, DeepEquals, []*ConnRef{NewConnRef(s.plugSelf, s.slot)})
+
+	conns, err = s.testRepo.Connections(s.slot.Snap.InstanceName())
+	c.Assert(err, IsNil)
+	c.Check(conns, DeepEquals, []*ConnRef{NewConnRef(s.plugSelf, s.slot)})
+}
+
 // Tests for Repository.DisconnectAll()
 
 func (s *RepositorySuite) TestDisconnectAll(c *C) {
 	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
 	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
-	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil)
+	_, err := s.testRepo.Connect(NewConnRef(s.plug, s.slot), nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	conns := []*ConnRef{NewConnRef(s.plug, s.slot)}
@@ -1382,7 +1542,7 @@ func (s *RepositorySuite) TestInterfacesSmokeTest(c *C) {
 	c.Assert(err, IsNil)
 	// After connecting the result is as expected
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = s.testRepo.Connect(connRef, nil, nil, nil)
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	ifaces := s.testRepo.Interfaces()
 	c.Assert(ifaces, DeepEquals, &Interfaces{
@@ -1443,7 +1603,7 @@ func (s *RepositorySuite) TestSnapSpecification(c *C) {
 
 	// Establish connection between plug and slot
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err = repo.Connect(connRef, nil, nil, nil)
+	_, err = repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	// Snaps should get static and connection-specific security now
@@ -1481,7 +1641,7 @@ func (s *RepositorySuite) TestSnapSpecificationFailureWithConnectionSnippets(c *
 	c.Assert(repo.AddPlug(s.plug), IsNil)
 	c.Assert(repo.AddSlot(s.slot), IsNil)
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err := repo.Connect(connRef, nil, nil, nil)
+	_, err := repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	spec, err := repo.SnapSpecification(testSecurity, s.plug.Snap.InstanceName())
@@ -1511,7 +1671,7 @@ func (s *RepositorySuite) TestSnapSpecificationFailureWithPermanentSnippets(c *C
 	c.Assert(repo.AddPlug(s.plug), IsNil)
 	c.Assert(repo.AddSlot(s.slot), IsNil)
 	connRef := NewConnRef(s.plug, s.slot)
-	_, err := repo.Connect(connRef, nil, nil, nil)
+	_, err := repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	spec, err := repo.SnapSpecification(testSecurity, s.plug.Snap.InstanceName())
@@ -1767,7 +1927,7 @@ func (s *AddRemoveSuite) TestRemoveSnapErrorsOnStillConnectedPlug(c *C) {
 	_, err = s.addSnap(c, testProducerYaml)
 	c.Assert(err, IsNil)
 	connRef := &ConnRef{PlugRef: PlugRef{Snap: "consumer", Name: "iface"}, SlotRef: SlotRef{Snap: "producer", Name: "iface"}}
-	_, err = s.repo.Connect(connRef, nil, nil, nil)
+	_, err = s.repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	err = s.repo.RemoveSnap("consumer")
 	c.Assert(err, ErrorMatches, "cannot remove connected plug consumer.iface")
@@ -1779,7 +1939,7 @@ func (s *AddRemoveSuite) TestRemoveSnapErrorsOnStillConnectedSlot(c *C) {
 	_, err = s.addSnap(c, testProducerYaml)
 	c.Assert(err, IsNil)
 	connRef := &ConnRef{PlugRef: PlugRef{Snap: "consumer", Name: "iface"}, SlotRef: SlotRef{Snap: "producer", Name: "iface"}}
-	_, err = s.repo.Connect(connRef, nil, nil, nil)
+	_, err = s.repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	err = s.repo.RemoveSnap("producer")
 	c.Assert(err, ErrorMatches, "cannot remove connected slot producer.iface")
@@ -1787,8 +1947,8 @@ func (s *AddRemoveSuite) TestRemoveSnapErrorsOnStillConnectedSlot(c *C) {
 
 type DisconnectSnapSuite struct {
 	testutil.BaseTest
-	repo   *Repository
-	s1, s2 *snap.Info
+	repo               *Repository
+	s1, s2, s2Instance *snap.Info
 }
 
 var _ = Suite(&DisconnectSnapSuite{})
@@ -1826,6 +1986,18 @@ slots:
 	c.Assert(err, IsNil)
 	err = s.repo.AddSnap(s.s2)
 	c.Assert(err, IsNil)
+	s.s2Instance = snaptest.MockInfo(c, `
+name: s2
+version: 0
+plugs:
+    iface-b:
+slots:
+    iface-a:
+`, nil)
+	s.s2Instance.InstanceKey = "instance"
+	c.Assert(err, IsNil)
+	err = s.repo.AddSnap(s.s2Instance)
+	c.Assert(err, IsNil)
 }
 
 func (s *DisconnectSnapSuite) TearDownTest(c *C) {
@@ -1840,7 +2012,7 @@ func (s *DisconnectSnapSuite) TestNotConnected(c *C) {
 
 func (s *DisconnectSnapSuite) TestOutgoingConnection(c *C) {
 	connRef := &ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "iface-a"}, SlotRef: SlotRef{Snap: "s2", Name: "iface-a"}}
-	_, err := s.repo.Connect(connRef, nil, nil, nil)
+	_, err := s.repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	// Disconnect s1 with which has an outgoing connection to s2
 	affected, err := s.repo.DisconnectSnap("s1")
@@ -1851,7 +2023,7 @@ func (s *DisconnectSnapSuite) TestOutgoingConnection(c *C) {
 
 func (s *DisconnectSnapSuite) TestIncomingConnection(c *C) {
 	connRef := &ConnRef{PlugRef: PlugRef{Snap: "s2", Name: "iface-b"}, SlotRef: SlotRef{Snap: "s1", Name: "iface-b"}}
-	_, err := s.repo.Connect(connRef, nil, nil, nil)
+	_, err := s.repo.Connect(connRef, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 	// Disconnect s1 with which has an incoming connection from s2
 	affected, err := s.repo.DisconnectSnap("s1")
@@ -1864,10 +2036,10 @@ func (s *DisconnectSnapSuite) TestCrossConnection(c *C) {
 	// This test is symmetric wrt s1 <-> s2 connections
 	for _, snapName := range []string{"s1", "s2"} {
 		connRef1 := &ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "iface-a"}, SlotRef: SlotRef{Snap: "s2", Name: "iface-a"}}
-		_, err := s.repo.Connect(connRef1, nil, nil, nil)
+		_, err := s.repo.Connect(connRef1, nil, nil, nil, nil, nil)
 		c.Assert(err, IsNil)
 		connRef2 := &ConnRef{PlugRef: PlugRef{Snap: "s2", Name: "iface-b"}, SlotRef: SlotRef{Snap: "s1", Name: "iface-b"}}
-		_, err = s.repo.Connect(connRef2, nil, nil, nil)
+		_, err = s.repo.Connect(connRef2, nil, nil, nil, nil, nil)
 		c.Assert(err, IsNil)
 		affected, err := s.repo.DisconnectSnap(snapName)
 		c.Assert(err, IsNil)
@@ -1876,6 +2048,22 @@ func (s *DisconnectSnapSuite) TestCrossConnection(c *C) {
 	}
 }
 
+func (s *DisconnectSnapSuite) TestParallelInstances(c *C) {
+	_, err := s.repo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "iface-a"}, SlotRef: SlotRef{Snap: "s2_instance", Name: "iface-a"}}, nil, nil, nil, nil, nil)
+	c.Assert(err, IsNil)
+	affected, err := s.repo.DisconnectSnap("s1")
+	c.Assert(err, IsNil)
+	c.Check(affected, testutil.Contains, "s1")
+	c.Check(affected, testutil.Contains, "s2_instance")
+
+	_, err = s.repo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s2_instance", Name: "iface-b"}, SlotRef: SlotRef{Snap: "s1", Name: "iface-b"}}, nil, nil, nil, nil, nil)
+	c.Assert(err, IsNil)
+	affected, err = s.repo.DisconnectSnap("s1")
+	c.Assert(err, IsNil)
+	c.Check(affected, testutil.Contains, "s1")
+	c.Check(affected, testutil.Contains, "s2_instance")
+}
+
 func contentPolicyCheck(plug *ConnectedPlug, slot *ConnectedSlot) (bool, error) {
 	return plug.Snap().Publisher.ID == slot.Snap().Publisher.ID, nil
 }
@@ -1968,39 +2156,58 @@ func (s *RepositorySuite) TestInfo(c *C) {
 	c.Assert(r.AddInterface(i3), IsNil)
 
 	// Add some test snaps.
-	s1 := snaptest.MockInfo(c, fmt.Sprintf(`
+	s1 := snaptest.MockInfo(c, `
 name: s1
 version: 0
 apps:
   s1:
     plugs: [i1, i2]
-`), nil)
+`, nil)
 	c.Assert(r.AddSnap(s1), IsNil)
 
-	s2 := snaptest.MockInfo(c, fmt.Sprintf(`
+	s2 := snaptest.MockInfo(c, `
 name: s2
 version: 0
 apps:
   s2:
     slots: [i1, i3]
-`), nil)
+`, nil)
 	c.Assert(r.AddSnap(s2), IsNil)
 
-	s3 := snaptest.MockInfo(c, fmt.Sprintf(`
+	s3 := snaptest.MockInfo(c, `
 name: s3
 version: 0
 type: os
 slots:
   i2:
-`), nil)
+`, nil)
 	c.Assert(r.AddSnap(s3), IsNil)
+	s3Instance := snaptest.MockInfo(c, `
+name: s3
+version: 0
+type: os
+slots:
+  i2:
+`, nil)
+	s3Instance.InstanceKey = "instance"
+	c.Assert(r.AddSnap(s3Instance), IsNil)
+	s4 := snaptest.MockInfo(c, `
+name: s4
+version: 0
+apps:
+  s1:
+    plugs: [i2]
+`, nil)
+	c.Assert(r.AddSnap(s4), IsNil)
 
 	// Connect a few things for the tests below.
-	_, err := r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i1"}, SlotRef: SlotRef{Snap: "s2", Name: "i1"}}, nil, nil, nil)
+	_, err := r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i1"}, SlotRef: SlotRef{Snap: "s2", Name: "i1"}}, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
-	_, err = r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i1"}, SlotRef: SlotRef{Snap: "s2", Name: "i1"}}, nil, nil, nil)
+	_, err = r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i1"}, SlotRef: SlotRef{Snap: "s2", Name: "i1"}}, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
-	_, err = r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i2"}, SlotRef: SlotRef{Snap: "s3", Name: "i2"}}, nil, nil, nil)
+	_, err = r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "i2"}, SlotRef: SlotRef{Snap: "s3", Name: "i2"}}, nil, nil, nil, nil, nil)
+	c.Assert(err, IsNil)
+	_, err = r.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s4", Name: "i2"}, SlotRef: SlotRef{Snap: "s3_instance", Name: "i2"}}, nil, nil, nil, nil, nil)
 	c.Assert(err, IsNil)
 
 	// Without any names or options we get the summary of all the interfaces.
@@ -2026,13 +2233,13 @@ slots:
 	// We can ask for a list of plugs.
 	infos = r.Info(&InfoOptions{Names: []string{"i2"}, Plugs: true})
 	c.Assert(infos, DeepEquals, []*Info{
-		{Name: "i2", Summary: "i2 summary", Plugs: []*snap.PlugInfo{s1.Plugs["i2"]}},
+		{Name: "i2", Summary: "i2 summary", Plugs: []*snap.PlugInfo{s1.Plugs["i2"], s4.Plugs["i2"]}},
 	})
 
 	// We can ask for a list of slots too.
 	infos = r.Info(&InfoOptions{Names: []string{"i2"}, Slots: true})
 	c.Assert(infos, DeepEquals, []*Info{
-		{Name: "i2", Summary: "i2 summary", Slots: []*snap.SlotInfo{s3.Slots["i2"]}},
+		{Name: "i2", Summary: "i2 summary", Slots: []*snap.SlotInfo{s3.Slots["i2"], s3Instance.Slots["i2"]}},
 	})
 
 	// We can also ask for only those interfaces that have connected plugs or slots.
@@ -2090,7 +2297,7 @@ func (s *RepositorySuite) TestBeforeConnectValidation(c *C) {
 	slotDynAttrs := map[string]interface{}{"attr1": "val1"}
 
 	policyCheck := func(plug *ConnectedPlug, slot *ConnectedSlot) (bool, error) { return true, nil }
-	conn, err := s.emptyRepo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "consumer"}, SlotRef: SlotRef{Snap: "s2", Name: "producer"}}, plugDynAttrs, slotDynAttrs, policyCheck)
+	conn, err := s.emptyRepo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "consumer"}, SlotRef: SlotRef{Snap: "s2", Name: "producer"}}, nil, plugDynAttrs, nil, slotDynAttrs, policyCheck)
 	c.Assert(err, IsNil)
 	c.Assert(conn, NotNil)
 
@@ -2125,7 +2332,7 @@ func (s *RepositorySuite) TestBeforeConnectValidationFailure(c *C) {
 
 	policyCheck := func(plug *ConnectedPlug, slot *ConnectedSlot) (bool, error) { return true, nil }
 
-	conn, err := s.emptyRepo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "consumer"}, SlotRef: SlotRef{Snap: "s2", Name: "producer"}}, plugDynAttrs, slotDynAttrs, policyCheck)
+	conn, err := s.emptyRepo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "consumer"}, SlotRef: SlotRef{Snap: "s2", Name: "producer"}}, nil, plugDynAttrs, nil, slotDynAttrs, policyCheck)
 	c.Assert(err, NotNil)
 	c.Assert(err, ErrorMatches, `cannot connect plug "consumer" of snap "s1": invalid plug`)
 	c.Assert(conn, IsNil)
@@ -2151,34 +2358,96 @@ func (s *RepositorySuite) TestBeforeConnectValidationPolicyCheckFailure(c *C) {
 		return false, fmt.Errorf("policy check failed")
 	}
 
-	conn, err := s.emptyRepo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "consumer"}, SlotRef: SlotRef{Snap: "s2", Name: "producer"}}, plugDynAttrs, slotDynAttrs, policyCheck)
+	conn, err := s.emptyRepo.Connect(&ConnRef{PlugRef: PlugRef{Snap: "s1", Name: "consumer"}, SlotRef: SlotRef{Snap: "s2", Name: "producer"}}, nil, plugDynAttrs, nil, slotDynAttrs, policyCheck)
 	c.Assert(err, NotNil)
 	c.Assert(err, ErrorMatches, `policy check failed`)
 	c.Assert(conn, IsNil)
 }
 
-type hotplugTestInterface struct{ InterfaceName string }
+func (s *RepositorySuite) TestConnection(c *C) {
+	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
+	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
 
-func (h *hotplugTestInterface) Name() string {
-	return h.InterfaceName
+	connRef := NewConnRef(s.plug, s.slot)
+
+	conn, err := s.testRepo.Connection(connRef)
+	c.Assert(err, ErrorMatches, `no connection from consumer:plug to producer:slot`)
+
+	_, err = s.testRepo.Connect(connRef, nil, nil, nil, nil, nil)
+	c.Assert(err, IsNil)
+
+	conn, err = s.testRepo.Connection(connRef)
+	c.Assert(err, IsNil)
+	c.Assert(conn.Plug.Name(), Equals, "plug")
+	c.Assert(conn.Slot.Name(), Equals, "slot")
+
+	conn, err = s.testRepo.Connection(&ConnRef{PlugRef: PlugRef{Snap: "a", Name: "b"}, SlotRef: SlotRef{Snap: "producer", Name: "slot"}})
+	c.Assert(err, ErrorMatches, `snap "a" has no plug named "b"`)
+
+	conn, err = s.testRepo.Connection(&ConnRef{PlugRef: PlugRef{Snap: "consumer", Name: "plug"}, SlotRef: SlotRef{Snap: "a", Name: "b"}})
+	c.Assert(err, ErrorMatches, `snap "a" has no slot named "b"`)
 }
 
-func (h *hotplugTestInterface) AutoConnect(plug *snap.PlugInfo, slot *snap.SlotInfo) bool {
-	return true
-}
+func (s *RepositorySuite) TestConnectWithStaticAttrs(c *C) {
+	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
+	c.Assert(s.testRepo.AddSlot(s.slot), IsNil)
 
-func (h *hotplugTestInterface) HotplugDeviceDetected(di *hotplug.HotplugDeviceInfo, spec *hotplug.Specification) error {
-	return nil
+	connRef := NewConnRef(s.plug, s.slot)
+
+	plugAttrs := map[string]interface{}{"foo": "bar"}
+	slotAttrs := map[string]interface{}{"boo": "baz"}
+	_, err := s.testRepo.Connect(connRef, plugAttrs, nil, slotAttrs, nil, nil)
+	c.Assert(err, IsNil)
+
+	conn, err := s.testRepo.Connection(connRef)
+	c.Assert(err, IsNil)
+	c.Assert(conn.Plug.Name(), Equals, "plug")
+	c.Assert(conn.Slot.Name(), Equals, "slot")
+	c.Assert(conn.Plug.StaticAttrs(), DeepEquals, plugAttrs)
+	c.Assert(conn.Slot.StaticAttrs(), DeepEquals, slotAttrs)
 }
 
 func (s *RepositorySuite) TestAllHotplugInterfaces(c *C) {
 	repo := NewRepository()
 	c.Assert(repo.AddInterface(&ifacetest.TestInterface{InterfaceName: "iface1"}), IsNil)
-	c.Assert(repo.AddInterface(&hotplugTestInterface{InterfaceName: "iface2"}), IsNil)
-	c.Assert(repo.AddInterface(&hotplugTestInterface{InterfaceName: "iface3"}), IsNil)
+	c.Assert(repo.AddInterface(&ifacetest.TestHotplugInterface{TestInterface: ifacetest.TestInterface{InterfaceName: "iface2"}}), IsNil)
+	c.Assert(repo.AddInterface(&ifacetest.TestHotplugInterface{TestInterface: ifacetest.TestInterface{InterfaceName: "iface3"}}), IsNil)
 
 	hi := repo.AllHotplugInterfaces()
 	c.Assert(hi, HasLen, 2)
 	c.Assert(hi[0].Name(), Equals, "iface2")
 	c.Assert(hi[1].Name(), Equals, "iface3")
 }
+
+func (s *RepositorySuite) TestHotplugMethods(c *C) {
+	c.Assert(s.testRepo.AddPlug(s.plug), IsNil)
+
+	coreSlot := &snap.SlotInfo{
+		Snap:       s.coreSnap,
+		Name:       "dummy-slot",
+		Interface:  "interface",
+		HotplugKey: "1234",
+	}
+	c.Assert(s.testRepo.AddSlot(coreSlot), IsNil)
+
+	slotInfo, err := s.testRepo.SlotForHotplugKey("interface", "1234")
+	c.Assert(err, IsNil)
+	c.Check(slotInfo, DeepEquals, coreSlot)
+
+	// no slot for device key 9999
+	slotInfo, err = s.testRepo.SlotForHotplugKey("interface", "9999")
+	c.Assert(err, IsNil)
+	c.Check(slotInfo, IsNil)
+
+	_, err = s.testRepo.Connect(NewConnRef(s.plug, coreSlot), nil, nil, nil, nil, nil)
+	c.Assert(err, IsNil)
+
+	conns, err := s.testRepo.ConnectionsForHotplugKey("interface", "1234")
+	c.Assert(err, IsNil)
+	c.Check(conns, DeepEquals, []*ConnRef{NewConnRef(s.plug, coreSlot)})
+
+	// no connections for device 9999
+	conns, err = s.testRepo.ConnectionsForHotplugKey("interface", "9999")
+	c.Assert(err, IsNil)
+	c.Check(conns, HasLen, 0)
+}
