@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2017 Canonical Ltd
+ * Copyright (C) 2017-2018 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -26,6 +26,7 @@ import (
 
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/strutil"
 )
 
 type entry struct {
@@ -41,7 +42,8 @@ type Specification struct {
 	entries  []entry
 	iface    string
 
-	securityTags []string
+	securityTags             []string
+	udevadmSubsystemTriggers []string
 }
 
 func (spec *Specification) addEntry(snippet, tag string) {
@@ -163,3 +165,27 @@ func (spec *Specification) AddPermanentSlot(iface interfaces.Interface, slot *sn
 	}
 	return nil
 }
+
+// Informs ReloadRules() to also do 'udevadm trigger <subsystem specific>'.
+// IMPORTANT: because there is currently no way to call TriggerSubsystem during
+// interface disconnect, TriggerSubsystem() should typically only by used in
+// UDevPermanentSlot since the rules are permanent until the snap is removed.
+func (spec *Specification) TriggerSubsystem(subsystem string) {
+	if subsystem == "" {
+		return
+	}
+
+	if strutil.ListContains(spec.udevadmSubsystemTriggers, subsystem) {
+		return
+	}
+	spec.udevadmSubsystemTriggers = append(spec.udevadmSubsystemTriggers, subsystem)
+}
+
+func (spec *Specification) TriggeredSubsystems() []string {
+	if len(spec.udevadmSubsystemTriggers) == 0 {
+		return nil
+	}
+	c := make([]string, len(spec.udevadmSubsystemTriggers))
+	copy(c, spec.udevadmSubsystemTriggers)
+	return c
+}
