@@ -22,12 +22,14 @@ package overlord
 import (
 	"time"
 
-	"github.com/ubuntu-core/snappy/osutil"
+	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/overlord/state"
 )
 
 type overlordStateBackend struct {
-	path         string
-	ensureBefore func(d time.Duration)
+	path           string
+	ensureBefore   func(d time.Duration)
+	requestRestart func(t state.RestartType)
 }
 
 func (osb *overlordStateBackend) Checkpoint(data []byte) error {
@@ -37,3 +39,7 @@ func (osb *overlordStateBackend) Checkpoint(data []byte) error {
 func (osb *overlordStateBackend) EnsureBefore(d time.Duration) {
 	osb.ensureBefore(d)
 }
+
+func (osb *overlordStateBackend) RequestRestart(t state.RestartType) {
+	osb.requestRestart(t)
+}
