@@ -36,6 +36,7 @@ import (
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/hookstate"
+	"github.com/snapcore/snapd/overlord/ifacestate"
 	"github.com/snapcore/snapd/overlord/patch"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
@@ -45,7 +46,9 @@ import (
 
 func TestOverlord(t *testing.T) { TestingT(t) }
 
-type overlordSuite struct{}
+type overlordSuite struct {
+	restoreBackends func()
+}
 
 var _ = Suite(&overlordSuite{})
 
@@ -54,14 +57,16 @@ func (ovs *overlordSuite) SetUpTest(c *C) {
 	dirs.SetRootDir(tmpdir)
 	dirs.SnapStateFile = filepath.Join(tmpdir, "test.json")
 	snapstate.CanAutoRefresh = nil
+	ovs.restoreBackends = ifacestate.MockSecurityBackends(nil)
 }
 
 func (ovs *overlordSuite) TearDownTest(c *C) {
 	dirs.SetRootDir("/")
+	ovs.restoreBackends()
 }
 
 func (ovs *overlordSuite) TestNew(c *C) {
-	restore := patch.Mock(42, nil)
+	restore := patch.Mock(42, 2, nil)
 	defer restore()
 
 	var configstateInitCalled bool
@@ -81,17 +86,25 @@ func (ovs *overlordSuite) TestNew(c *C) {
 	c.Check(o.HookManager(), NotNil)
 	c.Check(o.DeviceManager(), NotNil)
 	c.Check(o.CommandManager(), NotNil)
+	c.Check(o.SnapshotManager(), NotNil)
 	c.Check(configstateInitCalled, Equals, true)
 
+	o.InterfaceManager().DisableUDevMonitor()
+
 	s := o.State()
 	c.Check(s, NotNil)
 	c.Check(o.Engine().State(), Equals, s)
 
 	s.Lock()
 	defer s.Unlock()
-	var patchLevel int
+	var patchLevel, patchSublevel int
 	s.Get("patch-level", &patchLevel)
 	c.Check(patchLevel, Equals, 42)
+	s.Get("patch-sublevel", &patchSublevel)
+	c.Check(patchSublevel, Equals, 2)
+	var refreshPrivacyKey string
+	s.Get("refresh-privacy-key", &refreshPrivacyKey)
+	c.Check(refreshPrivacyKey, HasLen, 16)
 
 	// store is setup
 	sto := snapstate.Store(s)
@@ -100,7 +113,7 @@ func (ovs *overlordSuite) TestNew(c *C) {
 }
 
 func (ovs *overlordSuite) TestNewWithGoodState(c *C) {
-	fakeState := []byte(fmt.Sprintf(`{"data":{"patch-level":%d,"some":"data"},"changes":null,"tasks":null,"last-change-id":0,"last-task-id":0,"last-lane-id":0}`, patch.Level))
+	fakeState := []byte(fmt.Sprintf(`{"data":{"patch-level":%d,"patch-sublevel":%d,"some":"data","refresh-privacy-key":"0123456789ABCDEF"},"changes":null,"tasks":null,"last-change-id":0,"last-task-id":0,"last-lane-id":0}`, patch.Level, patch.Sublevel))
 	err := ioutil.WriteFile(dirs.SnapStateFile, fakeState, 0600)
 	c.Assert(err, IsNil)
 
@@ -124,6 +137,24 @@ func (ovs *overlordSuite) TestNewWithGoodState(c *C) {
 	c.Check(got, DeepEquals, expected)
 }
 
+func (ovs *overlordSuite) TestNewWithStateSnapmgrUpdate(c *C) {
+	fakeState := []byte(fmt.Sprintf(`{"data":{"patch-level":%d,"some":"data"},"changes":null,"tasks":null,"last-change-id":0,"last-task-id":0,"last-lane-id":0}`, patch.Level))
+	err := ioutil.WriteFile(dirs.SnapStateFile, fakeState, 0600)
+	c.Assert(err, IsNil)
+
+	o, err := overlord.New()
+	c.Assert(err, IsNil)
+
+	state := o.State()
+	c.Assert(err, IsNil)
+	state.Lock()
+	defer state.Unlock()
+
+	var refreshPrivacyKey string
+	state.Get("refresh-privacy-key", &refreshPrivacyKey)
+	c.Check(refreshPrivacyKey, HasLen, 16)
+}
+
 func (ovs *overlordSuite) TestNewWithInvalidState(c *C) {
 	fakeState := []byte(``)
 	err := ioutil.WriteFile(dirs.SnapStateFile, fakeState, 0600)
@@ -138,9 +169,13 @@ func (ovs *overlordSuite) TestNewWithPatches(c *C) {
 		s.Set("patched", true)
 		return nil
 	}
-	patch.Mock(1, map[int]func(*state.State) error{1: p})
+	sp := func(s *state.State) error {
+		s.Set("patched2", true)
+		return nil
+	}
+	patch.Mock(1, 1, map[int][]patch.PatchFunc{1: {p, sp}})
 
-	fakeState := []byte(fmt.Sprintf(`{"data":{"patch-level":0}}`))
+	fakeState := []byte(fmt.Sprintf(`{"data":{"patch-level":0, "patch-sublevel":0}}`))
 	err := ioutil.WriteFile(dirs.SnapStateFile, fakeState, 0600)
 	c.Assert(err, IsNil)
 
@@ -157,10 +192,17 @@ func (ovs *overlordSuite) TestNewWithPatches(c *C) {
 	c.Assert(err, IsNil)
 	c.Check(level, Equals, 1)
 
+	var sublevel int
+	c.Assert(state.Get("patch-sublevel", &sublevel), IsNil)
+	c.Check(sublevel, Equals, 1)
+
 	var b bool
 	err = state.Get("patched", &b)
 	c.Assert(err, IsNil)
 	c.Check(b, Equals, true)
+
+	c.Assert(state.Get("patched2", &b), IsNil)
+	c.Check(b, Equals, true)
 }
 
 type witnessManager struct {
@@ -211,6 +253,7 @@ func (ovs *overlordSuite) TestTrivialRunAndStop(c *C) {
 func (ovs *overlordSuite) TestUnknownTasks(c *C) {
 	o, err := overlord.New()
 	c.Assert(err, IsNil)
+	o.InterfaceManager().DisableUDevMonitor()
 
 	markSeeded(o)
 	// make sure we don't try to talk to the store
@@ -746,3 +789,29 @@ func (ovs *overlordSuite) TestRequestRestartHandler(c *C) {
 
 	c.Check(restartRequested, Equals, true)
 }
+
+func (ovs *overlordSuite) TestOverlordCanStandby(c *C) {
+	restoreIntv := overlord.MockEnsureInterval(10 * time.Millisecond)
+	defer restoreIntv()
+	o := overlord.Mock()
+	witness := &witnessManager{
+		state:          o.State(),
+		expectedEnsure: 3,
+		ensureCalled:   make(chan struct{}),
+	}
+	o.AddManager(witness)
+
+	// can only standby after loop ran once
+	c.Assert(o.CanStandby(), Equals, false)
+
+	o.Loop()
+	defer o.Stop()
+
+	select {
+	case <-witness.ensureCalled:
+	case <-time.After(2 * time.Second):
+		c.Fatal("Ensure calls not happening")
+	}
+
+	c.Assert(o.CanStandby(), Equals, true)
+}
