@@ -28,7 +28,6 @@ import (
 	"io"
 	"os"
 	"path/filepath"
-	"regexp"
 	"sort"
 	"time"
 
@@ -162,8 +161,10 @@ func Save(ctx context.Context, id uint64, si *snap.Info, cfg map[string]interfac
 	snapshot := &client.Snapshot{
 		SetID:    id,
 		Snap:     si.InstanceName(),
+		SnapID:   si.SnapID,
 		Revision: si.Revision,
 		Version:  si.Version,
+		Epoch:    si.Epoch,
 		Time:     time.Now(),
 		SHA3_384: make(map[string]string),
 		Size:     0,
@@ -225,15 +226,62 @@ func Save(ctx context.Context, id uint64, si *snap.Info, cfg map[string]interfac
 	return snapshot, nil
 }
 
+var isTesting = osutil.GetenvBool("SNAPPY_TESTING")
+
 func addDirToZip(ctx context.Context, snapshot *client.Snapshot, w *zip.Writer, username string, entry, dir string) error {
-	hasher := crypto.SHA3_384.New()
-	if exists, isDir, err := osutil.DirExists(dir); !exists || !isDir || err != nil {
-		if exists && !isDir {
-			logger.Noticef("Not saving %q in snapshot #%d of %q as it is not a directory.", dir, snapshot.SetID, snapshot.Snap)
-		}
+	parent, revdir := filepath.Split(dir)
+	exists, isDir, err := osutil.DirExists(parent)
+	if err != nil {
 		return err
 	}
-	parent, dir := filepath.Split(dir)
+	if exists && !isDir {
+		logger.Noticef("Not saving directories under %q in snapshot #%d of %q as it is not a directory.", parent, snapshot.SetID, snapshot.Snap)
+		return nil
+	}
+	if !exists {
+		logger.Debugf("Not saving directories under %q in snapshot #%d of %q as it is does not exist.", parent, snapshot.SetID, snapshot.Snap)
+		return nil
+	}
+	tarArgs := []string{
+		"--create",
+		"--sparse", "--gzip",
+		"--directory", parent,
+	}
+
+	noRev, noCommon := true, true
+
+	exists, isDir, err = osutil.DirExists(dir)
+	if err != nil {
+		return err
+	}
+	switch {
+	case exists && isDir:
+		tarArgs = append(tarArgs, revdir)
+		noRev = false
+	case exists && !isDir:
+		logger.Noticef("Not saving %q in snapshot #%d of %q as it is not a directory.", dir, snapshot.SetID, snapshot.Snap)
+	case !exists:
+		logger.Debugf("Not saving %q in snapshot #%d of %q as it is does not exist.", dir, snapshot.SetID, snapshot.Snap)
+	}
+
+	common := filepath.Join(parent, "common")
+	exists, isDir, err = osutil.DirExists(common)
+	if err != nil {
+		return err
+	}
+	switch {
+	case exists && isDir:
+		tarArgs = append(tarArgs, "common")
+		noCommon = false
+	case exists && !isDir:
+		logger.Noticef("Not saving %q in snapshot #%d of %q as it is not a directory.", common, snapshot.SetID, snapshot.Snap)
+	case !exists:
+		logger.Debugf("Not saving %q in snapshot #%d of %q as it is does not exist.", common, snapshot.SetID, snapshot.Snap)
+	}
+
+	if noCommon && noRev {
+		return nil
+	}
 
 	archiveWriter, err := w.CreateHeader(&zip.FileHeader{Name: entry})
 	if err != nil {
@@ -241,16 +289,16 @@ func addDirToZip(ctx context.Context, snapshot *client.Snapshot, w *zip.Writer,
 	}
 
 	var sz sizer
+	hasher := crypto.SHA3_384.New()
 
-	cmd := maybeRunuserCommand(username,
-		"tar",
-		"--create",
-		"--sparse", "--gzip",
-		"--directory", parent, dir, "common")
-	cmd.Env = []string{"GZIP=-9 -n"}
+	cmd := tarAsUser(username, tarArgs...)
 	cmd.Stdout = io.MultiWriter(archiveWriter, hasher, &sz)
-	matchCounter := &strutil.MatchCounter{Regexp: regexp.MustCompile(".*"), N: 1}
+	matchCounter := &strutil.MatchCounter{N: 1}
 	cmd.Stderr = matchCounter
+	if isTesting {
+		matchCounter.N = -1
+		cmd.Stderr = io.MultiWriter(os.Stderr, matchCounter)
+	}
 	if err := osutil.RunWithContext(ctx, cmd); err != nil {
 		matches, count := matchCounter.Matches()
 		if count > 0 {
