@@ -21,6 +21,8 @@ package snap
 
 var (
 	ValidateSocketName           = validateSocketName
+	ValidateDescription          = validateDescription
+	ValidateTitle                = validateTitle
 	InfoFromSnapYamlWithSideInfo = infoFromSnapYamlWithSideInfo
 )
 
